/**CHeaderFile*****************************************************************

  FileName    [ cce.h ]

  PackageName [ cce ]

  Synopsis    [ C++ documentation extractor ]

  Description [ A high-level interface to a simplified C++ parser ]

  Author      [ Stephen A. Edwards <sedwards@eecs.berkeley.edu> ]

  Copyright   []

  Revision    []

******************************************************************************/

#ifndef _CCE
#define _CCE

#include <stdio.h>
#include "tcl.h"


/**Type************************************************************************

  Synopsis    [ an object ]

  Description [ An array of void *.  Each is a slot in the object, typecast
		as necessary. ]

******************************************************************************/
typedef void ** Cce_Object;


/**Constants******************************************************************

  Synopsis    [ The possible slot types ]

  Description [ ]

******************************************************************************/
#define CCE_OBJECT 0
#define CCE_STRING 1
#define CCE_INTEGER 2

#define CCE_PRIVATE 0
#define CCE_PROTECTED 1
#define CCE_PUBLIC 2

#define CCE_VIRTUAL 1
#define CCE_CONST 2
#define CCE_STATIC 4
#define CCE_INLINE 8
#define CCE_UNSIGNED 16
#define CCE_SHORT 32
#define CCE_LONG 64
#define CCE_ARRAY 128
#define CCE_PURE 256

/**Variable********************************************************************
  Synopsis    [ An array of pointers to all objects in the system,
		indexed by ID ]

  Description [ This array grows as necessary.  numObjects is the
		current number of objects, objectTableSize is the size
		of the memory allocated for it.
		]

******************************************************************************/
extern Cce_Object * cce_objectPointers;
extern int cce_numObjects;

/**Variable********************************************************************
  Synopsis    [ Objects for each class ]
******************************************************************************/
extern Cce_Object cce_classClass;
extern Cce_Object cce_slotClass;

#include "cceObjectHeader.include"

/**Macro***********************************************************************
  Synopsis     [ Read an object slot ]
  SideEffects  []
******************************************************************************/
#define /* Cce_Object */ Cce_ObjectSlotReadObject( /* Cce_Object */ o, \
						  /* int */ s) \
   (Cce_Object) (o)[s]

/**Macro***********************************************************************
  Synopsis     [ Read a string slot ]
  SideEffects  []
******************************************************************************/
#define /* char * */ Cce_ObjectSlotReadString( /* Cce_Object */ o, \
					      /* int */ s) \
   (char *) (o)[s]

/**Macro***********************************************************************
  Synopsis     [ Read an integer slot ]
  SideEffects  []
******************************************************************************/
#define /* int */ Cce_ObjectSlotReadInt( /* Cce_Object */ o, \
					/* int */ s) \
   (int) (o)[s]

/**Macro***********************************************************************
  Synopsis     [ Write a slot ]
  SideEffects  []
******************************************************************************/
#define Cce_ObjectSlotWrite( /* Cce_Object */ o, \
			    /* int */ s, v ) \
   (o)[s] = (void *) (v)

/*
 * Slot numbers for the built-in classes
 */

#define CCE_OBJECT_CLASS 0
#define CCE_OBJECT_ID 1
#define CCE_OBJECT_NEXT 2

#define CCE_CLASS_NAME 3
#define CCE_CLASS_NUM_OBJECTS 4
#define CCE_CLASS_NUM_SLOTS 5
#define CCE_CLASS_SLOTS 6

#define CCE_SLOT_NAME 3
#define CCE_SLOT_TYPE 4

/*
 * Slots common to most AST objects
 */
#define CCE_ASTOBJECT_LINE 3
#define CCE_ASTOBJECT_FILE 4


/**Macro***********************************************************************
  Synopsis     [ Read the class of an object ]
  SideEffects  []
******************************************************************************/
#define Cce_ObjectReadClass( /* Cce_Object */ o) \
	Cce_ObjectSlotReadObject(o, CCE_OBJECT_CLASS)

/**Macro***********************************************************************
  Synopsis     [ Write the class of an object ]
  SideEffects  []
******************************************************************************/
#define Cce_ObjectWriteClass( /* Cce_Object */ o, /* Cce_Object */ v) \
	Cce_ObjectSlotWrite(o, CCE_OBJECT_CLASS, v)


/**Macro***********************************************************************
  Synopsis     [ Read the id of an object ]
  SideEffects  []
******************************************************************************/
#define /* int */ Cce_ObjectReadId( /* Cce_Object */ o) \
	Cce_ObjectSlotReadInt(o, CCE_OBJECT_ID)

/**Macro***********************************************************************
  Synopsis     [ Write the id of an object ]
  SideEffects  []
******************************************************************************/
#define Cce_ObjectWriteId( /* Cce_Object */ o, /* int */ v) \
	Cce_ObjectSlotWrite(o, CCE_OBJECT_ID, v)

/**Macro***********************************************************************
  Synopsis     [ Read the next object of an object ]
  SideEffects  []
******************************************************************************/
#define /* Cce_Object */ Cce_ObjectReadNext( /* Cce_Object */ o) \
	Cce_ObjectSlotReadObject(o, CCE_OBJECT_NEXT)

/**Macro***********************************************************************
  Synopsis     [ Write the next object of an object ]
  SideEffects  []
******************************************************************************/
#define Cce_ObjectWriteNext( /* Cce_Object */ o, /* Cce_Object */ v) \
	Cce_ObjectSlotWrite(o, CCE_OBJECT_NEXT, v)

/**Macro***********************************************************************
  Synopsis     [ Read the number of slots in a class ]
  SideEffects  []
******************************************************************************/
#define /* int */ Cce_ClassReadNumSlots( /* Cce_Object */ o) \
	Cce_ObjectSlotReadInt(o, CCE_CLASS_NUM_SLOTS )

/**Macro***********************************************************************
  Synopsis     [ Read the slots in a class ]
  SideEffects  []
******************************************************************************/
#define /* Cce_Object */ Cce_ClassReadSlots( /* Cce_Object */ o) \
	Cce_ObjectSlotReadObject(o, CCE_CLASS_SLOTS )

/**Macro***********************************************************************
  Synopsis     [ Read the name of a slot ]
  SideEffects  []
******************************************************************************/
#define /* char * */ Cce_SlotReadName( /* Cce_Object */ o) \
	Cce_ObjectSlotReadString(o, CCE_SLOT_NAME )

/**Macro***********************************************************************
  Synopsis     [ Read the type of a slot ]
  SideEffects  []
******************************************************************************/
#define /* int */ Cce_SlotReadType( /* Cce_Object */ o) \
	Cce_ObjectSlotReadInt(o, CCE_SLOT_TYPE)


/**Macro***********************************************************************
  Synopsis     [ Write the line number of an AST object ]
  SideEffects  []
******************************************************************************/
#define Cce_AstObjectWriteLine( /* Cce_Object */ o, /* int */ l) \
	Cce_ObjectSlotWrite(o, CCE_ASTOBJECT_LINE, l)

/**Macro***********************************************************************
  Synopsis     [ Write the filename of an AST object ]
  SideEffects  []
******************************************************************************/
#define Cce_AstObjectWriteFilename( /* Cce_Object */ o, /* char * */ n) \
	Cce_ObjectSlotWrite(o, CCE_ASTOBJECT_FILE, n)

/**Macro***********************************************************************

  Synopsis     [ Return the object with the given ID ]

  Description  [ Returns NULL if the ID is invalid. ]

  SideEffects  []

  SeeAlso      [ CceRegisterObject ]

******************************************************************************/
#define Cce_ObjectOfId(id) \
  ((id) < 0 || (id) >= cce_numObjects ? 0 : cce_objectPointers[(id)])

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN int Cce_Init(Tcl_Interp * interp);
EXTERN void Cce_MemberListSetAccess(Cce_Object list, int access);
EXTERN void Cce_MemberListSetClass(Cce_Object list, char * className);
EXTERN int Cce_ParseFile(FILE * file, char * filename, Cce_Object * object, Tcl_DString * parseErrors);
EXTERN Cce_Object Cce_ObjectNew(Cce_Object class);
EXTERN int Cce_ClassAddSlot(Cce_Object class, char * name, int type);
EXTERN Cce_Object Cce_ClassNew(char * name);
EXTERN void Cce_ObjectsInitialize();
EXTERN Cce_Object Cce_ObjectAppendObject(Cce_Object object, Cce_Object list);
EXTERN int Cce_ClassFindNamedSlot(Cce_Object class, char * slotName, int * type);

/**AutomaticEnd***************************************************************/

#endif /* _CCE */
